//--------------------------------------------------------------------
// TUCN, Computer Science Department
// Input/Output Systems and Peripheral Devices
//--------------------------------------------------------------------
// http://users.utcluj.ro/~baruch/en/pages/teaching/inputoutput-systems/laboratory.php
//--------------------------------------------------------------------
// File:		TestCom1DT-e.cpp
// Changed: 	16.02.2024
//--------------------------------------------------------------------
// Testing the COM1 serial port
//--------------------------------------------------------------------
// - Application example to illustrate port accesses
// - The MarvinHW driver is used for port accesses
// - The DrawText function is used for displaying the messages
//--------------------------------------------------------------------
// - Win API functions used:
//   CreateFile, DrawText, GetClientRect, GetDC, GetLastError, 
//   MessageBox, ReleaseDC
//--------------------------------------------------------------------

#include <windows.h>
#include "Hw.h"
#include "ComDef-e.h"

#define NLIN 500								// number of lines in the display window
#define NCOL 240								// number of columns in the display window

// Global variables
char szBuffer[NLIN][NCOL];						// buffer for the window contents
int  cLine;										// number of current line in the display buffer

// Declarations of external functions
void DisplWindow(HWND hWnd);

//--------------------------------------------------------------------
// Function TestCom1DT
//--------------------------------------------------------------------
//
// Function:	Tests the existence of the COM1 serial port
//
// Parameters:	hWnd - Handle to the application window
//
// Returns:		0 - Operation completed successfully
//				1 - Error initializing the Hw driver
//				2 - Error opening the COM1 port
//
//--------------------------------------------------------------------

int TestCom1DT(HWND hWnd)
{
	DWORD dwLastError;							// code of last error
	HANDLE hFile;
	BYTE  byVal;								// value read from port
	BOOL  bSucc;
	int   i;

	char szMes0[] = "Error initializing the Hw driver\nCode: 0x%08X";
	char szMes1[] = "Testing the COM1 serial port using the MarvinHW driver";
	char szMes2[] = "Serial port with base address 0x%03X";
	char szMes3[] = "Value written to LCR register: 0xAA";
	char szMes4[] = "Value written to LCR register: 0x55";
	char szMes5[] = "Value read from LCR register: 0x%02X";
	char szMes6[] = "The port exists and works correctly";
	char szMes7[] = "The port does not exist or does not work correctly";
	char szMes8[] = "Error opening the COM1 port\nCode: 0x%08X";

	// Initialize the Hw library
	if (!HwOpen()) {
		dwLastError = GetLastError();
		wsprintf(szBuffer[0], szMes0, dwLastError);
		MessageBox(NULL, szBuffer[0], "HwOpen", MB_ICONSTOP);
		return 1;
	}

	// Open the COM1 port
	hFile = CreateFile("COM1", GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (hFile == INVALID_HANDLE_VALUE) {
		dwLastError = GetLastError();
		wsprintf(szBuffer[0], szMes8, dwLastError);
		MessageBox(NULL, szBuffer[0], "CreateFile", MB_ICONSTOP);
		return 2;
	}

	// Clear the display buffer and the window contents
	for (i = 0; i < NLIN; i++) {
		memset(szBuffer[i], ' ', NCOL);
	}
	cLine = 1;

	// Copy the start message into the display buffer and display the message
	wsprintf(szBuffer[cLine], szMes1);
	cLine += 2;
	DisplWindow(hWnd);

	// Test the COM1 serial port and display the messages
	wsprintf(szBuffer[cLine++], szMes2, COM1);
	cLine++;
	__outp(COM1_LCR, 0xAA);						// write the value 0xAA to the LCR register
	byVal = __inp(COM1_LCR);					// read the value written
	wsprintf(szBuffer[cLine++], szMes3);
	wsprintf(szBuffer[cLine++], szMes5, byVal);	// display the value read from the LCR register
	cLine++;
	if (byVal == 0xAA)
		bSucc = TRUE;
	else
		bSucc = FALSE;
	__outp(COM1_LCR, 0x55);						// write the value 0x55 to the LCR register
	byVal = __inp(COM1_LCR);					// read the value written
	wsprintf(szBuffer[cLine++], szMes4);
	wsprintf(szBuffer[cLine++], szMes5, byVal);	// display the value read from the LCR register
	cLine++;
	if (bSucc && (byVal == 0x55))
		wsprintf(szBuffer[cLine++], szMes6);
	else
		wsprintf(szBuffer[cLine++], szMes7);

	// Set the coordinates of the display area and display the messages
	DisplWindow(hWnd);

	CloseHandle(hFile);
	HwClose();
	return 0;
}
